unit JCommon;

interface

uses Windows, Graphics, Classes, Controls, Forms, StdCtrls;

type
  TJStyle = (jsNormal, jsFlat);
  TJTextStyle = (tsNormal, tsEllipsis, tsFilePath);
  TJTextInfo = record
    Alignment: TAlignment;
    Enabled: Boolean;
    Layout: TTextLayout;
    Font: TFont;
    ShowAccelChar: Boolean;
    Style: TJTextStyle;
    Text: String;
  end;


procedure DoDrawFrame(ACanvas: TCanvas; ARect: TRect; AStyle: TJStyle;
  ADown: Boolean);
function CalckTextRect(ACanvas: TCanvas; ARect: TRect; Info: TJTextInfo): TRect;
procedure DoDrawText(ACanvas: TCanvas; ARect: TRect; Info: TJTextInfo);
function GetCheckRect(ARect: TRect): TRect;
procedure DrawCheckBox(Canvas: TCanvas; ARect: TRect; Checked: Boolean);
procedure ShrinkRect(var ARect: TRect; n: Integer);

implementation

procedure DoDrawFrame(ACanvas: TCanvas; ARect: TRect; AStyle: TJStyle;
  ADown: Boolean);
var
  Style: LongInt;
begin
  case AStyle of
    jsNormal: begin
      Style := 0;
      if ADown then
        Style := BF_FLAT;
      DrawEdge(ACanvas.Handle, ARect, EDGE_RAISED, BF_TOPLEFT or Style);
      DrawEdge(ACanvas.Handle, ARect, EDGE_RAISED, BF_BOTTOMRIGHT or Style);
    end;
    jsFlat: begin
      if not ADown then
      begin
        DrawEdge(ACanvas.Handle, ARect, BDR_RAISEDINNER, BF_TOPLEFT);
        DrawEdge(ACanvas.Handle, ARect, BDR_RAISEDINNER, BF_BOTTOMRIGHT);
      end else
        begin
          DrawEdge(ACanvas.Handle, ARect, BDR_SUNKENINNER, BF_TOPLEFT);
          DrawEdge(ACanvas.Handle, ARect, BDR_SUNKENINNER, BF_BOTTOMRIGHT);
        end;
    end;
  end;
end;

function CalckTextRect(ACanvas: TCanvas; ARect: TRect; Info: TJTextInfo): TRect;
const
  Alignments: array[TAlignment] of Word = (DT_LEFT, DT_RIGHT, DT_CENTER);
var
  Style: LongInt;
  R: TRect;
begin
  if Info.Text <> '' then
  begin
    ACanvas.Font.Assign(Info.Font);
    Style := DT_SINGLELINE or Alignments[Info.Alignment] or DT_EXPANDTABS or DT_NOCLIP;
    if not Info.ShowAccelChar then
      Style := Style or DT_NOPREFIX;
    R := ARect;
    DrawText(ACanvas.Handle, PChar(Info.Text), Length(Info.Text), R, Style or DT_CALCRECT);
    Result := R;
  end else
    begin
      Result := R;
      Result.Right := Result.Left;
    end;
end;

procedure _DrawText(ACanvas: TCanvas; ARect: TRect; Info: TJTextInfo);
const
  Alignments: array[TAlignment] of Word = (DT_LEFT, DT_RIGHT, DT_CENTER);
var
  Style: LongInt;
  R: TRect;
  h: Integer;
begin
  Style := DT_SINGLELINE or Alignments[Info.Alignment] or DT_EXPANDTABS{ or DT_NOCLIP};
  case Info.Style of
    tsEllipsis: Style := Style or DT_END_ELLIPSIS;
    tsFilePath: begin
      ARect.Right := ARect.Right - ACanvas.TextWidth('...');
      Style := Style or DT_PATH_ELLIPSIS;
    end;
  end;
  if not Info.ShowAccelChar then
    Style := Style or DT_NOPREFIX;
  R := ARect;
  h := DrawText(ACanvas.Handle, PChar(Info.Text), Length(Info.Text), R, Style or DT_CALCRECT);
  R := ARect;
  case Info.Layout of
    tlBottom: R.Top := R.Bottom - h - 1;
    tlCenter: begin
      R.Top := R.Top + (R.Bottom - R.Top - h) div 2;
      R.Bottom := R.Top + h;
    end;
  end;
  DrawText(ACanvas.Handle, PChar(Info.Text), Length(Info.Text), R, Style);
end;

procedure DoDrawText(ACanvas: TCanvas; ARect: TRect; Info: TJTextInfo);
begin
  ACanvas.Font.Assign(Info.Font);
  ACanvas.Brush.Style := bsClear;
  if not Info.Enabled then
  begin
    OffsetRect(ARect, 1, 1);
    ACanvas.Font.Color := clBtnHighlight;
    _DrawText(ACanvas, ARect, Info);
    OffsetRect(ARect, -1, -1);
    ACanvas.Font.Color := clBtnShadow;
    _DrawText(ACanvas, ARect, Info);
  end else
    _DrawText(ACanvas, ARect, Info);
end;

procedure ShrinkRect(var ARect: TRect; n: Integer);
begin
  ARect.Top := ARect.Top + n;
  ARect.Left := ARect.Left + n;
  ARect.Right := ARect.Right - n;
  ARect.Bottom := ARect.Bottom - n;
end;

function GetCheckRect(ARect: TRect): TRect;
const
  CKBOX_SIZE = 13;
var
  X, Y: Integer;
begin
  X := ((ARect.Right - ARect.Left) - CKBOX_SIZE) div 2;
  Y := ((ARect.Bottom - ARect.Top) - CKBOX_SIZE) div 2;
  Result := Rect(ARect.Left + X, ARect.Top + Y, ARect.Left + X + CKBOX_SIZE, ARect.Top + Y + CKBOX_SIZE);
end;


procedure DrawCheckBox(Canvas: TCanvas; ARect: TRect; Checked: Boolean);
begin
  with Canvas do
  begin
    ARect := GetCheckRect(ARect);
    Brush.Style := bsSolid;
    Brush.Color := clWindow;
    Pen.Style := psSolid;
    Pen.Color := clBtnShadow;
    Rectangle(ARect.Left, ARect.Top, ARect.Right, ARect.Bottom);

    if Checked then
    begin
      Pen.Width := 1;
      Pen.Color := clBlack;
      InflateRect(ARect, -3, -3);
      MoveTo(ARect.Left, ARect.Top + 2);
      LineTo(ARect.Left + 3, ARect.Bottom - 2);
      MoveTo(ARect.Left, ARect.Top + 3);
      LineTo(ARect.Left + 3, ARect.Bottom - 1);
      MoveTo(ARect.Left, ARect.Top + 4);
      LineTo(ARect.Left + 3, ARect.Bottom);

      MoveTo(ARect.Left + 3, ARect.Top + 3);
      LineTo(ARect.Right, ARect.Top - 1);
      MoveTo(ARect.Left + 3, ARect.Top + 4);
      LineTo(ARect.Right, ARect.Top);
      MoveTo(ARect.Left + 3, ARect.Top + 5);
      LineTo(ARect.Right, ARect.Top + 1);
    end;
  end;
end;

end.
